@@ -33,8 +33,9 @@
 
 #define CONFIG_BASENAME "monitors.xml"
 
-/* In version 0 of the config file format, we had several <configuration> toplevel elements
- * and no explicit version number.  So, the filed looked like
+/* In version 0 of the config file format, we had several <configuration>
+ * toplevel elements and no explicit version number.  So, the filed looked
+ * like
  *
  *   <configuration>
  *     ...
@@ -43,9 +44,9 @@
  *     ...
  *   </configuration>
  *
- * Since version 1 of the config file, the file has a toplevel <monitors> element to group
- * all the configurations.  That element has a "version" attribute which is an integer.
- * So, the file looks like this:
+ * Since version 1 of the config file, the file has a toplevel <monitors>
+ * element to group all the configurations.  That element has a "version"
+ * attribute which is an integer. So, the file looks like this:
  *
  *   <monitors version="1">
  *     <configuration>
@@ -65,7 +66,7 @@ static gboolean parse_file_gmarkup (const gchar *file,
 
 typedef struct CrtcAssignment CrtcAssignment;
 
-static void             crtc_assignment_apply (CrtcAssignment   *assign);
+static gboolean         crtc_assignment_apply (CrtcAssignment   *assign);
 static CrtcAssignment  *crtc_assignment_new   (GnomeRRScreen    *screen,
 					       GnomeOutputInfo **outputs);
 static void             crtc_assignment_free  (CrtcAssignment   *assign);
@@ -463,6 +464,7 @@ gnome_rr_config_new_current (GnomeRRScreen *screen)
     int i;
     int clone_width = -1;
     int clone_height = -1;
+    int last_x;
 
     g_return_val_if_fail (screen != NULL, NULL);
 
@@ -536,7 +538,7 @@ gnome_rr_config_new_current (GnomeRRScreen *screen)
 				clone_width = output->width;
 				clone_height = output->height;
 			} else if (clone_width == output->width &&
-					clone_height == output->height) {
+				   clone_height == output->height) {
 				config->clone = TRUE;
 			}
 		}
@@ -588,6 +590,34 @@ gnome_rr_config_new_current (GnomeRRScreen *screen)
     
     config->outputs = (GnomeOutputInfo **)g_ptr_array_free (a, FALSE);
 
+    /* Walk the outputs computing the right-most edge of all
+     * lit-up displays
+     */
+    last_x = 0;
+    for (i = 0; config->outputs[i] != NULL; ++i)
+    {
+	GnomeOutputInfo *output = config->outputs[i];
+
+	if (output->on)
+	{
+	    last_x = MAX (last_x, output->x + output->width);
+	}
+    }
+
+    /* Now position all off displays to the right of the
+     * on displays
+     */
+    for (i = 0; config->outputs[i] != NULL; ++i)
+    {
+	GnomeOutputInfo *output = config->outputs[i];
+
+	if (output->connected && !output->on)
+	{
+	    output->x = last_x;
+	    last_x = output->x + output->width;
+	}
+    }
+    
     g_assert (gnome_rr_config_match (config, config));
     
     return config;
@@ -716,6 +746,42 @@ output_match (GnomeOutputInfo *output1, GnomeOutputInfo *output2)
     return TRUE;
 }
 
+static gboolean
+output_equal (GnomeOutputInfo *output1, GnomeOutputInfo *output2)
+{
+    g_assert (output1 != NULL);
+    g_assert (output2 != NULL);
+
+    if (!output_match (output1, output2))
+	return FALSE;
+
+    if (output1->on != output2->on)
+	return FALSE;
+
+    if (output1->on)
+    {
+	if (output1->width != output2->width)
+	    return FALSE;
+	
+	if (output1->height != output2->height)
+	    return FALSE;
+	
+	if (output1->rate != output2->rate)
+	    return FALSE;
+	
+	if (output1->x != output2->x)
+	    return FALSE;
+	
+	if (output1->y != output2->y)
+	    return FALSE;
+	
+	if (output1->rotation != output2->rotation)
+	    return FALSE;
+    }
+
+    return TRUE;
+}
+
 static GnomeOutputInfo *
 find_output (GnomeRRConfig *config, const char *name)
 {
@@ -732,6 +798,9 @@ find_output (GnomeRRConfig *config, const char *name)
     return NULL;
 }
 
+/* Match means "these configurations apply to the same hardware
+ * setups"
+ */
 gboolean
 gnome_rr_config_match (GnomeRRConfig *c1, GnomeRRConfig *c2)
 {
@@ -750,6 +819,28 @@ gnome_rr_config_match (GnomeRRConfig *c1, GnomeRRConfig *c2)
     return TRUE;
 }
 
+/* Equal means "the configurations will result in the same
+ * modes being set on the outputs"
+ */
+gboolean
+gnome_rr_config_equal (GnomeRRConfig  *c1,
+		       GnomeRRConfig  *c2)
+{
+    int i;
+
+    for (i = 0; c1->outputs[i] != NULL; ++i)
+    {
+	GnomeOutputInfo *output1 = c1->outputs[i];
+	GnomeOutputInfo *output2;
+
+	output2 = find_output (c2, output1->name);
+	if (!output2 || !output_equal (output1, output2))
+	    return FALSE;
+    }
+    
+    return TRUE;
+}
+
 static GnomeOutputInfo **
 make_outputs (GnomeRRConfig *config)
 {
@@ -811,21 +902,6 @@ gnome_rr_config_applicable (GnomeRRConfig  *configuration,
     return result;
 }
 
-static GnomeRRConfig *
-gnome_rr_config_find (GnomeRRConfig **haystack,
-		    GnomeRRConfig  *needle)
-{
-    int i;
-
-    for (i = 0; haystack[i] != NULL; ++i)
-    {
-	if (gnome_rr_config_match (haystack[i], needle))
-	    return haystack[i];
-    }
-
-    return NULL;
-}
-
 /* Database management */
 
 static gchar *
@@ -1007,13 +1083,69 @@ gnome_rr_config_save (GnomeRRConfig *configuration, GError **err)
     return result;
 }
 
-static gboolean
-apply_configuration (GnomeRRConfig *conf, GnomeRRScreen *screen)
+static GnomeRRConfig *
+gnome_rr_config_copy (GnomeRRConfig *config)
+{
+    GnomeRRConfig *copy = g_new0 (GnomeRRConfig, 1);
+    int i;
+    GPtrArray *array = g_ptr_array_new ();
+
+    copy->clone = config->clone;
+    
+    for (i = 0; config->outputs[i] != NULL; ++i)
+	g_ptr_array_add (array, output_copy (config->outputs[i]));
+
+    g_ptr_array_add (array, NULL);
+    copy->outputs = (GnomeOutputInfo **)g_ptr_array_free (array, FALSE);
+
+    return copy;
+}
+
+GnomeRRConfig *
+gnome_rr_config_new_stored (GnomeRRScreen *screen)
+{
+    GnomeRRConfig *current;
+    GnomeRRConfig **configs;
+    GnomeRRConfig *result;
+
+    if (!screen)
+	return NULL;
+    
+    current = gnome_rr_config_new_current (screen);
+    
+    configs = configurations_read (NULL); /* NULL_GError */
+
+    result = NULL;
+    if (configs)
+    {
+	int i;
+	
+	for (i = 0; configs[i] != NULL; ++i)
+	{
+	    if (gnome_rr_config_match (configs[i], current))
+	    {
+		result = gnome_rr_config_copy (configs[i]);
+		break;
+	    }
+	}
+
+	configurations_free (configs);
+    }
+
+    gnome_rr_config_free (current);
+    
+    return result;
+}
+
+gboolean
+gnome_rr_config_apply (GnomeRRConfig *config,
+		       GnomeRRScreen *screen)
 {
     CrtcAssignment *assignment;
     GnomeOutputInfo **outputs;
+    gboolean result = FALSE;
 
-    outputs = make_outputs (conf);
+    outputs = make_outputs (config);
 
     assignment = crtc_assignment_new (screen, outputs);
 
@@ -1021,50 +1153,64 @@ apply_configuration (GnomeRRConfig *conf, GnomeRRScreen *screen)
     
     if (assignment)
     {
-	crtc_assignment_apply (assignment);
+	if (crtc_assignment_apply (assignment))
+	    result = TRUE;
 	    
 	crtc_assignment_free (assignment);
 
-	return TRUE;
+	gdk_flush ();
     }
 
-    return FALSE;
+    return result;
 }
 
 gboolean
 gnome_rr_config_apply_stored (GnomeRRScreen *screen)
 {
-    GnomeRRConfig **configs = configurations_read (NULL); /* NULL-GError */
-    GnomeRRConfig *current;
-    GnomeRRConfig *found;
-    gboolean result = TRUE;
+    GnomeRRConfig *stored;
 
     if (!screen)
 	return FALSE;
-    
-    gnome_rr_screen_refresh (screen);
-    
-    current = gnome_rr_config_new_current (screen);
-    if (configs)
-    {
-	if ((found = gnome_rr_config_find (configs, current)))
-	{
-	    apply_configuration (found, screen);
-	    result = TRUE;
-	}
-	else
-	{
-	    result = FALSE;
-	}
-	
-	configurations_free (configs);
-    }
-	
-    gnome_rr_config_free (current);
 
-    return result;
+    gnome_rr_screen_refresh (screen);
+
+    stored = gnome_rr_config_new_stored (screen);
+
+    if (stored)
+    {
+	gnome_rr_config_apply (stored, screen);
+
+	gnome_rr_config_free (stored);
+	
+	return TRUE;
+    }
+    else
+    {
+	return FALSE;
+    }
 }
 
+static gboolean
+has_similar_mode (GnomeRROutput *output, GnomeRRMode *mode)
+{
+    int i;
+    GnomeRRMode **modes = gnome_rr_output_list_modes (output);
+    int width = gnome_rr_mode_get_width (mode);
+    int height = gnome_rr_mode_get_height (mode);
+
+    for (i = 0; modes[i] != NULL; ++i)
+    {
+	GnomeRRMode *m = modes[i];
+
+	if (gnome_rr_mode_get_width (m) == width	&&
+	    gnome_rr_mode_get_height (m) == height)
+	{
+	    return TRUE;
+	}
+    }
+
+    return FALSE;
+}
 
 /*
  * CRTC assignment
@@ -1104,16 +1250,14 @@ can_clone (CrtcInfo *info,
 }
 
 static gboolean
-crtc_assignment_assign (CrtcAssignment *assign,
-			GnomeRRCrtc         *crtc,
-			GnomeRRMode         *mode,
-			int             x,
-			int             y,
-			GnomeRRRotation      rotation,
-			GnomeRROutput       *output)
+crtc_assignment_assign (CrtcAssignment   *assign,
+			GnomeRRCrtc      *crtc,
+			GnomeRRMode      *mode,
+			int               x,
+			int               y,
+			GnomeRRRotation   rotation,
+			GnomeRROutput    *output)
 {
-    /* FIXME: We should reject stuff that is outside the screen ranges */
-    
     CrtcInfo *info = g_hash_table_lookup (assign->info, crtc);
 
     if (!gnome_rr_crtc_can_drive_output (crtc, output) ||
@@ -1221,98 +1365,6 @@ crtc_is_rotated (GnomeRRCrtc *crtc)
     return FALSE;
 }
 
-static void
-crtc_assignment_apply (CrtcAssignment *assign)
-{
-    GList *active_crtcs = g_hash_table_get_keys (assign->info);
-    GnomeRRCrtc **all_crtcs = gnome_rr_screen_list_crtcs (assign->screen);
-    GList *list;
-    int width, height;
-    int i;
-    int min_width, max_width, min_height, max_height;
-    int width_mm, height_mm;
-
-    /* Compute size of the screen */
-    width = height = 1;
-    for (list = active_crtcs; list != NULL; list = list->next)
-    {
-	GnomeRRCrtc *crtc = list->data;
-	CrtcInfo *info = g_hash_table_lookup (assign->info, crtc);
-	int w, h;
-
-	w = gnome_rr_mode_get_width (info->mode);
-	h = gnome_rr_mode_get_height (info->mode);
-	
-	if (mode_is_rotated (info))
-	{
-	    int tmp = h;
-	    h = w;
-	    w = tmp;
-	}
-	
-	width = MAX (width, info->x + w);
-	height = MAX (height, info->y + h);
-    }
-    g_list_free (active_crtcs);
-
-    gnome_rr_screen_get_ranges (
-	assign->screen, &min_width, &max_width, &min_height, &max_height);
-
-    width = MAX (min_width, width);
-    width = MIN (max_width, width);
-    height = MAX (min_height, height);
-    height = MIN (max_height, height);
-    
-    /* Turn off all crtcs currently displaying outside the new screen */
-    for (i = 0; all_crtcs[i] != NULL; ++i)
-    {
-	GnomeRRCrtc *crtc = all_crtcs[i];
-	GnomeRRMode *mode = gnome_rr_crtc_get_current_mode (crtc);
-	int x, y;
-
-	if (mode)
-	{
-	    int w, h;
-	    gnome_rr_crtc_get_position (crtc, &x, &y);
-
-	    w = gnome_rr_mode_get_width (mode);
-	    h = gnome_rr_mode_get_height (mode);
-	    
-	    if (crtc_is_rotated (crtc))
-	    {
-		int tmp = h;
-		h = w;
-		w = tmp;
-	    }
-	    
-	    if (x + w > width || y + h > height)
-		gnome_rr_crtc_set_config (crtc, 0, 0, NULL, GNOME_RR_ROTATION_0, NULL, 0);
-	}
-    }
-
-    /* Turn off all CRTC's that are not in the assignment */
-    for (i = 0; all_crtcs[i] != NULL; ++i)
-    {
-	GnomeRRCrtc *crtc = all_crtcs[i];
-	
-	if (!g_hash_table_lookup (assign->info, crtc))
-	    gnome_rr_crtc_set_config (crtc, 0, 0, NULL, GNOME_RR_ROTATION_0, NULL, 0);
-    }
-
-    /* The 'physical size' of an X screen is meaningless if that screen
-     * can consist of many monitors. So just pick a size that make the
-     * dpi 96.
-     *
-     * Firefox and Evince apparently believe what X tells them.
-     */
-    width_mm = (width / 96.0) * 25.4 + 0.5;
-    height_mm = (height / 96.0) * 25.4 + 0.5;
-    
-    gnome_rr_screen_set_size (assign->screen, width, height, width_mm, height_mm);
-
-    g_hash_table_foreach (assign->info, configure_crtc, NULL);
-}
-
 /* Check whether the given set of settings can be used
  * at the same time -- ie. whether there is an assignment
  * of CRTC's to outputs.
@@ -1388,6 +1440,43 @@ crtc_info_free (CrtcInfo *info)
     g_free (info);
 }
 
+static void
+get_required_virtual_size (CrtcAssignment *assign, int *width, int *height)
+{
+    GList *active_crtcs = g_hash_table_get_keys (assign->info);
+    GList *list;
+    int d;
+
+    if (!width)
+	width = &d;
+    if (!height)
+	height = &d;
+    
+    /* Compute size of the screen */
+    *width = *height = 1;
+    for (list = active_crtcs; list != NULL; list = list->next)
+    {
+	GnomeRRCrtc *crtc = list->data;
+	CrtcInfo *info = g_hash_table_lookup (assign->info, crtc);
+	int w, h;
+
+	w = gnome_rr_mode_get_width (info->mode);
+	h = gnome_rr_mode_get_height (info->mode);
+	
+	if (mode_is_rotated (info))
+	{
+	    int tmp = h;
+	    h = w;
+	    w = tmp;
+	}
+	
+	*width = MAX (*width, info->x + w);
+	*height = MAX (*height, info->y + h);
+    }
+
+    g_list_free (active_crtcs);
+}
+
 static CrtcAssignment *
 crtc_assignment_new (GnomeRRScreen *screen, GnomeOutputInfo **outputs)
 {
@@ -1398,14 +1487,106 @@ crtc_assignment_new (GnomeRRScreen *screen, GnomeOutputInfo **outputs)
 
     if (real_assign_crtcs (screen, outputs, assignment))
     {
+	int width, height;
+	int min_width, max_width, min_height, max_height;
+
+	get_required_virtual_size (assignment, &width, &height);
+
+	gnome_rr_screen_get_ranges (
+	    screen, &min_width, &max_width, &min_height, &max_height);
+    
+	if (width < min_width || width > max_width ||
+	    height < min_height || height > max_height)
+	{
+	    goto fail;
+	}
+
 	assignment->screen = screen;
 	
 	return assignment;
     }
-    else
-    {
-	crtc_assignment_free (assignment);
     
-	return NULL;
+fail:
+    crtc_assignment_free (assignment);
+    
+    return NULL;
+}
+
+static gboolean
+crtc_assignment_apply (CrtcAssignment *assign)
+{
+    GnomeRRCrtc **all_crtcs = gnome_rr_screen_list_crtcs (assign->screen);
+    int width, height;
+    int i;
+    int min_width, max_width, min_height, max_height;
+    int width_mm, height_mm;
+    gboolean success = TRUE;
+
+    /* Compute size of the screen */
+    get_required_virtual_size (assign, &width, &height);
+
+    gnome_rr_screen_get_ranges (
+	assign->screen, &min_width, &max_width, &min_height, &max_height);
+
+    /* We should never get here if the dimensions don't fit in the virtual size,
+     * but just in case we do, fix it up.
+     */
+    width = MAX (min_width, width);
+    width = MIN (max_width, width);
+    height = MAX (min_height, height);
+    height = MIN (max_height, height);
+    
+    /* Turn off all crtcs that are currently displaying outside the new screen,
+     * or are not used in the new setup
+     */
+    for (i = 0; all_crtcs[i] != NULL; ++i)
+    {
+	GnomeRRCrtc *crtc = all_crtcs[i];
+	GnomeRRMode *mode = gnome_rr_crtc_get_current_mode (crtc);
+	int x, y;
+
+	if (mode)
+	{
+	    int w, h;
+	    gnome_rr_crtc_get_position (crtc, &x, &y);
+
+	    w = gnome_rr_mode_get_width (mode);
+	    h = gnome_rr_mode_get_height (mode);
+	    
+	    if (crtc_is_rotated (crtc))
+	    {
+		int tmp = h;
+		h = w;
+		w = tmp;
+	    }
+	    
+	    if (x + w > width || y + h > height || !g_hash_table_lookup (assign->info, crtc))
+	    {
+		if (!gnome_rr_crtc_set_config (crtc, 0, 0, NULL, GNOME_RR_ROTATION_0, NULL, 0))
+		{
+		    success = FALSE;
+		    break;
+		}
+		
+	    }
+	}
     }
+
+    /* The 'physical size' of an X screen is meaningless if that screen
+     * can consist of many monitors. So just pick a size that make the
+     * dpi 96.
+     *
+     * Firefox and Evince apparently believe what X tells them.
+     */
+    width_mm = (width / 96.0) * 25.4 + 0.5;
+    height_mm = (height / 96.0) * 25.4 + 0.5;
+
+    if (success)
+    {
+	gnome_rr_screen_set_size (assign->screen, width, height, width_mm, height_mm);
+	
+	g_hash_table_foreach (assign->info, configure_crtc, NULL);
+    }
+
+    return success;
 }
